# XV6-RICV
---

### 项目简述
XV6-RICV是一款基于XV6研发的RISC-V架构宏内核操作系统。
开发过程中，部分代码设计思想参考了Linux内核源码及[uCore-Tutorial-Guide-2023A](https://gitlab.eduxiji.net/educg-group-14238-914330/853476998-2958)文档

---
### 项目改进
与XV6相比，我们做了以下工作：
+ 修改了内存模块，添加了内存共享功能，并且实现了对应的系统调用，还增加了用户态程序用来验证
+ 调度对象由进程更改为线程，实现了Linux标准下的细粒度clone、execve和futex等17个进程相关的系统调用；
+ 支持EXT4文件系统的挂载，实现了openat、readv和sendfile等33个文件相关的系统调用；
+ 支持内核态的堆空间分配，实现了mmap、shmget和shmat等8个内存相关的系统调用；
+ 部分支持Linux信号机制的处理，实现了tkill、rt_sigaction和rt_sigprocmask等6个信号相关的系统调用；
+ 支持Socket通信；
+ 支持VisionFive2的板载运行。

仍需改进的地方：

+ 目前内核并不稳定，对内存的释放存在问题；
+ 对大内存的高效管理；
+ 更好的进程/线程调度机制；
+ IO仍然较慢，需要进行特殊优化。

---

### 项目架构
├── bootloader
│   ├── bootasm.S
│   └── bootmain.c
├── kernel
│   ├── asm.h
│   ├── bio.c
│   ├── buf.h
│   ├── console.c
│   ├── date.h
│   ├── defs.h
│   ├── elf.h
│   ├── entry.S
│   ├── exec.c
│   ├── fcntl.h
│   ├── file.c
│   ├── file.h
│   ├── fs.c
│   ├── fs.h
│   ├── ide.c
│   ├── ioapic.c
│   ├── kalloc.c
│   ├── kbd.c
│   ├── kbd.h
│   ├── lapic.c
│   ├── log.c
│   ├── main.c
│   ├── memide.c
│   ├── memlayout.h
│   ├── mmu.h
│   ├── mp.c
│   ├── mp.h
│   ├── param.h
│   ├── proc.c
│   ├── spinlock.h
│   ├── sleeplock.h
│   ├── string.c
│   ├── syscall.c
│   ├── timer.c
│   ├── trap.c
│   ├── uart.c
│   └── vm.c
├── user
│   ├── cat.c
│   ├── echo.c
│   ├── forktest.c
│   ├── grep.c
│   ├── init.c
│   ├── kill.c
│   ├── ln.c
│   ├── ls.c
│   ├── mkdir.c
│   ├── rm.c
│   ├── sh.c
│   ├── usertests.c
│   ├── wc.c
│   └── zombie.c
└── tools
    └── mkfs.c

---

### 编译与运行

#### 环境准备

交叉编译工具链：riscv64-unknown-linux-gnu-

运行环境：Qemu-7.0.0 for Riscv64


#### 编译
make all进行编译

#### 运行
make run运行内核

#### 调试
make debug运行debug模式的内核
make gdb进入gdb调试

---

### 文档
- 内核启动
- 设备管理
- 文件系统
- 系统通用
- 异常中断
- 内存管理
- 进程管理
- 用户态程序
- 测试日志
- 遇到的问题及解决方案
  
---