# 主要内容

基于xv6操作系统，修改了内存模块，添加了内存共享功能，并且实现了对应的系统调用，还增加了用户态程序用来验证

## **新系统调用**：

- **`share_mem`**：该系统调用允许父进程注册和初始化共享内存结构。每个注册的共享内存结构需要一个唯一的名称（限制为20个字符），一个指向起始地址的指针，以及共享内存的大小。每个进程最多可以注册10个共享内存结构，以限制资源使用。注册时会检查是否存在重复名称、参数有效性，以及是否超过允许的注册数。有关错误的处理包括返回指定错误码，例如：同名的共享内存存在、参数不正确或达到共享结构的限制。
- **`get_shared`**：此系统调用用于子进程以访问父进程注册的共享内存。通过提供共享结构的名称，子进程可以获得一个指向共享内存的有效指针，借助此指针，子进程可以直接操作共享的内存空间。这需要精确处理以确保只有在确实可以访问的情况下才会设定指针并避免非法的内存访问。

## **系统修改**：

- **`fork`**：在实现`fork`系统调用时，子进程必须继承父进程的共享内存结构。为了追踪父进程的页面目录，子进程的`proc`结构中需要有一个专用字段，用于存储父进程的内存管理信息。这保证了即便在虚拟地址空间被子进程复制的情况下，仍然可以访问这些共享内存。
- **`exec`**：执行`exec`调用时，系统需要重新配置新进程的虚拟内存结构。在存粹的新进程内存映像中，1GB的限制用于常规使用。而通过从1GB开始的地址，将父进程的共享内存映射至子进程的虚拟空间。这种映射使子进程在`exec`调用中依然可以访问父进程共享的内存，并且需要调整新的虚拟地址来保持一致性。
- **进程终止**：必须确保所有进程（父进程及其子进程）结束时能够正确地释放使用的所有内存资源。这不仅包括进程本身分配的内存，也包括那些通过共享机制获得的内存。这样可以避免出现未释放的内存，影响系统的长期运行稳定性。

## **用户程序**：

- **`primestart`**：作为父进程，`primestart`的主要职责是设定共享内存并管理子进程的生命周期。这涉及创建多个共享内存段，比如一个用于存储整数的数组，一个用于追踪发现的素数位置的计数器，以及一个用于管理指令的指示器。创建这些共享结构后，它启动两个子进程：`primecom`用于处理用户输入，`primecalc`则负责计算素数。父进程将等待这两个进程完成他们的任务后，适时退出。
- **`primecom`**：此进程负责用户交互，通过命令控制共享内存和`primecalc`的行为。用户可以通过`primecom`发送指令来查看特定的素数、暂停或恢复计算，甚至结束计算过程。命令如`prime<n>`和`latest`直接从共享内存获取数据，而`pause`、`resume`、`end`等命令会更改共享内存中的状态，影响`primecalc`的计算行为。
- **`primecalc`**：专注于素数的计算，从2开始逐一检查每个数字是否为素数，如果是，就将其记录在共享数组中。通过共享的命令指示器，它能够接收到来自`primecom`的命令。在计算每一个数字的过程中，它会不断检查命令指示器的值，若为`pause`，则暂时停止计算，直到收到`resume`命令，而接收到`end`命令后会进行清理并自行终止