## 内存管理

xv6的内存管理分为物理和虚拟两个模块，主要涉及以下文件：
├── vm.c // 虚拟内存管理
├── kalloc.c // 物理内存分配
└── memlayout.h // 内存布局定义

### 物理内存管理

物理内存管理主要负责内存空间管理和动态内存分配。

1. **内存空间管理**

系统采用空闲链表的方式管理物理内存。每个空闲页面作为一个链表节点,结构如下:

​```c
struct run {
struct run next;
};
struct {
struct spinlock lock;
struct run freelist;
} kmem;
​```
​
2. **动态内存分配**

- `kalloc()`: 从空闲链表中分配一页物理内存
- `kfree()`: 释放一页物理内存并加入空闲链表
- `kinit()`: 初始化物理内存管理,建立空闲页面链表

### 虚拟内存管理

xv6采用二级页表机制实现虚拟内存管理。

#### 内存布局

```
KERNBASE(0x80000000) ---> +------------------+
                          |     Kernel       |
                          +------------------+ 
SHAREBASE(0x40000000) --> |  Shared Memory  |
                          +------------------+
                          |   User Memory    |
0x0 ------------------> +------------------+
```

- 内核空间: KERNBASE(2GB)以上
- 共享内存: SHAREBASE(1GB)到KERNBASE  
- 用户空间: 0到SHAREBASE

#### 页表操作

1. **页表创建与映射**

- `setupkvm()`: 创建内核页表
- `copyuvm()`: 复制用户进程页表
- `mappages()`: 建立虚拟地址到物理地址的映射

2. **共享内存操作**

- `mappa2va()`: 将物理页面映射到指定的虚拟地址
- `linkphysaddr()`: 链接共享内存的物理地址

3. **页表查找与释放**

- `walkpgdir()`: 在页表中查找PTE
- `deallocuvm()`: 释放用户虚拟内存
- `freevm()`: 释放整个页表

#### 共享内存管理

1. **共享内存结构**

​```
c
struct shared {
char name[SHAREDNAME]; // 共享内存名称
char memstart; // 起始地址
int sz; // 大小
};
​```

2. **系统调用**

- `share_mem(char *name, void *addr, int size)`: 注册共享内存
- `get_shared(char *name, void **addr)`: 获取共享内存

3. **进程间共享实现**

- fork时继承父进程的共享内存结构
- exec时重新映射共享内存
- exit时释放共享内存资源

#### 地址转换过程

虚拟地址到物理地址的转换通过以下步骤:

1. 从虚拟地址提取页目录索引(PDX)
2. 从页目录获取页表地址
3. 从虚拟地址提取页表索引(PTX)
4. 从页表获取物理页面地址
5. 加上偏移得到最终物理地址

相关宏定义:
​​c
\# define PDX(va) (((uint)(va) >> PDXSHIFT) & 0x3FF)
\# define PTX(va) (((uint)(va) >> PTXSHIFT) & 0x3FF)
\# define PTE_ADDR(pte) ((uint)(pte) & ~0xFFF)

