# 操作系统启动流程
操作系统常见启动流程如下
![操作系统启动流程](image1.png)

---



### BIOS 引导
- 在 `start` 函数中，代码首先禁用了 BIOS 启用的中断。
- 清零了数据段寄存器 `DS`、`ES` 和 `SS`，确保不使用任何之前的段。

### A20 地址线的启用
- 早期的 x86 机器上，A20 地址线被禁用，这会导致访问超过 1MB 地址的内存出现问题。代码通过向 I/O 端口发送命令来启用 A20 地址线，以便支持超过 1MB 的内存访问。

### 从实模式切换到保护模式
- 在引导加载程序中，使用了 GDT（全局描述符表）来从实模式（16 位模式）切换到保护模式（32 位模式）。
  - 通过设置 CR0 寄存器的保护模式标志（PE bit），将 CPU 从 16 位的实模式切换到 32 位的保护模式。
  - 首先通过 `lgdt` 指令加载 GDT 描述符。
  - 然后使用 `ljmp`（长跳转）指令跳转到保护模式的代码段，从而完成从实模式到保护模式的转换。

### 程序的加载
- 在保护模式下，内核引导代码从硬盘加载内核程序。这个过程通过 `bootmain()` 函数实现。
  - `bootmain()` 通过 `readseg()` 函数将 ELF 格式的内核从硬盘读取到内存中。
  - 在读取内核文件时，首先通过 `readseg()` 读取一个页面（4096 字节），并检查是否为 ELF 文件。如果文件是 ELF 格式，接着读取程序头表（Program Header Table）中的各个段。
  - 每个程序段的内容从磁盘读取到指定的物理地址。如果该段的文件大小小于内存大小（即 BSS 段），则剩余部分会用零填充。

### 调用 ELF 入口点
- ELF 文件的入口点是由 ELF 头中的 `entry` 字段指定的。代码通过 `entry = (void(*)(void))(V2P(elf->entry));` 获得入口点地址，并通过 `entry()` 跳转到内核的入口，开始执行内核代码。

### 启动过程
- ####   主CPU 的启动（main 函数）
main 函数是系统启动时主 CPU 执行的入口。它完成了系统初始化的各个关键步骤，首先为系统分配和初始化内存，接着初始化各种硬件设备和内核资源，然后启动其他处理器。实现了多核系统的引导过程。
```cpp
int main(void)
{
    kinit1(end, P2V(4*1024*1024)); // 初始化物理页面分配器
    kvmalloc(); // 初始化内核页表

    mpinit(); // 检测其他处理器
    lapicinit(); // 初始化本地中断控制器
    seginit(); // 初始化段描述符
    picinit(); // 禁用PIC
    ioapicinit(); // 初始化I/O中断控制器
    consoleinit(); // 初始化控制台

    uartinit(); // 初始化串口
    pinit(); // 初始化进程表
    tvinit(); // 初始化中断向量表

    binit(); // 初始化缓冲区缓存
    fileinit(); // 初始化文件表

    ideinit(); // 初始化磁盘驱动
    startothers(); // 启动其他处理器
    kinit2(P2V(4*1024*1024), P2V(PHYSTOP)); // 继续初始化内存分配器
    userinit(); // 启动第一个用户进程
    mpmain(); // 完成主 CPU 的初始化
}
```
- ####   启动其他处理器的入口（mpenter 函数）
每个启动的 CPU 都会跳转到 mpenter 函数，在这里，AP 会执行一些初始化工作，然后跳转到 mpmain()。mpmain 是所有 CPU 执行的共同初始化函数，它完成了每个 CPU 的初始化工作，并最终开始调度进程。
```cpp
static void mpmain(void)
{
    cprintf("cpu%d: starting%d\n", cpuid(), cpuid());
    idtinit(); // 加载中断描述符表
    xchg(&(mycpu()->started), 1); // 标记该 CPU 已经启动
    scheduler(); // 启动进程调度器
}
```
- #### 启动 GDT 和页表（entrypgdir 和 bootgdt）
entrypgdir 是为每个处理器（包括主 CPU 和其他处理器）初始化的页目录，它定义了虚拟地址到物理地址的映射。在多核系统中，主 CPU 使用自己的页目录，其他处理器使用共享的 entrypgdir