## 1.日志事务管理
日志系统中的每一个事务包含多个文件系统调用的更新，并且日志的提交是同步的，直到所有的文件系统调用完成。日志系统将每次更新记录在磁盘上，并确保只有在所有系统调用完成后才会提交事务，从而避免了部分提交的风险。

**begin_op()** 和 **end_op()** 用于标记文件系统操作的开始和结束。日志系统通过 begin_op() 和 end_op() 来支持并发文件系统操作。每个操作都会在开始时增加一个“正在进行的操作计数”，确保所有的操作在日志提交之前都能顺利完成。这样设计避免了在多个文件系统调用并发执行时，出现数据不一致的情况。
在操作开始时，begin_op() 会通过加锁和判断当前是否有足够的日志空间来决定是否开始新的操作。如果日志空间不足，则会等待日志提交完成后再进行操作。
```
if(log.committing){
			sleep(&log, &log.lock);
		} else if(log.lh.n + (log.outstanding+1)*MAXOPBLOCKS > LOGSIZE){
			// this op might exhaust log space; wait for commit.
			sleep(&log, &log.lock);
		} else {
			log.outstanding += 1;
			release(&log.lock);
			break;
		}
```
**end_op()** 会在操作结束时检查当前是否是最后一个未完成的操作，如果是，则触发日志的提交。
```
if (do _ commit) {
 commit(); // 提交日志
 acquire(& log. lock);
 log.committing = 0;
 wakeup(& log);
 release(& log. lock);
}
```
**commit()** 和 **write_log()** 在事务完成时将日志数据写入磁盘，并确保所有的修改被持久化。
```
void commit() {
 if (log. lh. n > 0) {
 write _ log(); // 将修改写入日志
 write _ head(); // 写入头部, 正式提交
 install _ trans(); // 将事务数据安装到目标位置
 log. lh. n = 0;
 write _ head(); // 清空事务日志
    }
 }
```
## 2.日志恢复机制
**recover_from_log()** 通过读取日志头部并回放已提交的日志，恢复未完成的事务。这个过程确保了即使在崩溃后，磁盘数据也能恢复到一致状态。
```
 recover _ from _ log(void)
{
 read _ head(); // 读取日志头部信息
 install _ trans(); // 如果事务已经提交, 则从日志恢复数据到文件系统
 log. lh. n = 0; // 清空日志头中的已提交事务数量
 write _ head(); // 清除日志, 重置日志头
}
```
## 3. 日志的写入和安装
日志的写入过程是同步的，通过 commit() 方法，系统保证了日志的同步提交。当一个事务包含多个文件系统调用时，只有在所有调用都完成后，事务才会被提交到磁盘，这样确保了磁盘数据的一致性。在一个事务提交后，系统会将日志中保存的块数据写回原始位置，从而确保文件系统的状态正确。