## 目录结构
├── user
│   ├── grep.c           // 用户态grep程序
│   └── sh.c             // 用户态shell程序

---
## 基础数据类型
```
// 基本类型定义
typedef unsigned int   uint;
typedef unsigned short ushort;
typedef unsigned char  uchar;

// 系统相关类型
typedef uint64 pde_t;
typedef uint64 *pagetable_t;
```
---
## 用户空间程序
### Shell实现
Shell是用户与操作系统交互的命令行界面，主要功能：
  - 命令解析
  - 管道支持
  - 重定向支持
  - 后台任务
### Grep实现
Grep是一个文本搜索工具，主要特点：
  - 支持正则表达式匹配
  - 支持文件输入
  - 支持管道输入
  - 可配置的输出格式
  
---
  
### 相较于xv6的改进
1. 添加了一个用于追踪发现的素数位置的计数器，以及一个用于管理指令的指示器。
2. 增加了进行用于用户交互的功能，实现了通过命令行来控制共享内存。用户可以自行发送指令查看素数的计算过程。
3. 增加了素数计数模块
---
#### 具体改进细节
- **`primestart`**：作为父进程，`primestart`的主要职责是设定共享内存并管理子进程的生命周期。这涉及创建多个共享内存段，比如一个用于存储整数的数组，一个用于追踪发现的素数位置的计数器，以及一个用于管理指令的指示器。创建这些共享结构后，它启动两个子进程：`primecom`用于处理用户输入，`primecalc`则负责计算素数。父进程将等待这两个进程完成他们的任务后，适时退出。
- **`primecom`**：此进程负责用户交互，通过命令控制共享内存和`primecalc`的行为。用户可以通过`primecom`发送指令来查看特定的素数、暂停或恢复计算，甚至结束计算过程。命令如`prime<n>`和`latest`直接从共享内存获取数据，而`pause`、`resume`、`end`等命令会更改共享内存中的状态，影响`primecalc`的计算行为。
```
	for(int i=0 ; ; ++i){
		
		if(i >= MAXSHARED)
			break;
	
		if(strncmp(name, curproc->shared[i].name, SHAREDNAME) == 0){

			*addr = curproc->shared[i].memstart;
			if(strncmp("primecom", curproc->name, SHAREDNAME && primecom) == 0)
				wakeup(&curproc->parent);		
			primecom = 0;		

			return 0;
		}
	}

	if(strncmp("primecom", curproc->name, SHAREDNAME) == 0 && primecom)
		wakeup(&curproc->parent);
	primecom = 0;
		
	return -2;
  }
```
**`primecalc`**：专注于素数的计算，从2开始逐一检查每个数字是否为素数，如果是，就将其记录在共享数组中。通过共享的命令指示器，它能够接收到来自`primecom`的命令。在计算每一个数字的过程中，它会不断检查命令指示器的值，若为`pause`，则暂时停止计算，直到收到`resume`命令，而接收到`end`命令后会进行清理并自行终止。
```cpp
if (child_a == 0) {

		exec("/bin/primecalc", argvcalc);
		exit();

	}else {
	
		child_b = fork();

		if (child_b == 0) {
			exec("/bin/primecom", argvcom);
			exit();
		} else {
		  	wpid2 = wait(); // Wait for primecom
		}
	}
	
	wpid1 = wait(); // Wait for primecom for primecalc
						
	free(primes); 
	
	exit();
```
---
### 进程间通信
#### 管道
管道是进程间通信的基本机制：

  - 单向数据流
  - 读写同步
  - 缓冲区管理
  
我们模拟了一个环形缓冲区，并了通过 pipealloc 分配管道，pipewrite 和 piperead 实现对管道的读写操作，pipeclose 管理管道的关闭。并使用锁来保证数据的同步读写。自旋锁和睡眠锁被用来保证对共享资源（管道缓冲区）的同步访问。
##### 结构
```
struct pipe {
    struct spinlock lock;    // 用于保证管道的并发访问的自旋锁
    char data[PIPESIZE];     // 用于存储管道数据的缓冲区
    uint nread;              // 已读的字节数
    uint nwrite;             // 已写的字节数
    int readopen;            // 读端是否仍然打开
    int writeopen;           // 写端是否仍然打开
};
```
